package java.net;

import java.io.IOException;
import java.io.FileDescriptor;

public abstract class DatagramSocketImpl implements SocketOptions
{
    protected int localPort;
    protected FileDescriptor fd;
    DatagramSocket socket;
    
    public DatagramSocketImpl() {
    }
    
    int dataAvailable() {
        return 0;
    }
    
    void setDatagramSocket(DatagramSocket socket) {
        this.socket = socket;
    }
    
    DatagramSocket getDatagramSocket() {
        return this.socket;
    }
    
    protected abstract void create() throws SocketException;
    
    protected abstract void bind(int p0, InetAddress p1) throws SocketException;
    
    protected abstract void send(DatagramPacket p0) throws IOException;
    
    protected void connect(InetAddress inetAddress, int n) throws SocketException {
    }
    
    protected void disconnect() {
    }
    
    protected abstract int peek(InetAddress p0) throws IOException;
    
    protected abstract int peekData(DatagramPacket p0) throws IOException;
    
    protected abstract void receive(DatagramPacket p0) throws IOException;
    
    @Deprecated
    protected abstract void setTTL(byte p0) throws IOException;
    
    @Deprecated
    protected abstract byte getTTL() throws IOException;
    
    protected abstract void setTimeToLive(int p0) throws IOException;
    
    protected abstract int getTimeToLive() throws IOException;
    
    protected abstract void join(InetAddress p0) throws IOException;
    
    protected abstract void leave(InetAddress p0) throws IOException;
    
    protected abstract void joinGroup(SocketAddress p0, NetworkInterface p1) throws IOException;
    
    protected abstract void leaveGroup(SocketAddress p0, NetworkInterface p1) throws IOException;
    
    protected abstract void close();
    
    protected int getLocalPort() {
        return this.localPort;
    }
    
     <T> void setOption(SocketOption<T> socketOption, T t) throws IOException {
        if (socketOption == StandardSocketOptions.SO_SNDBUF) {
            this.setOption(4097, t);
        }
        else if (socketOption == StandardSocketOptions.SO_RCVBUF) {
            this.setOption(4098, t);
        }
        else if (socketOption == StandardSocketOptions.SO_REUSEADDR) {
            this.setOption(4, t);
        }
        else if (socketOption == StandardSocketOptions.IP_TOS) {
            this.setOption(3, t);
        }
        else if (socketOption == StandardSocketOptions.IP_MULTICAST_IF && this.getDatagramSocket() instanceof MulticastSocket) {
            this.setOption(31, t);
        }
        else if (socketOption == StandardSocketOptions.IP_MULTICAST_TTL && this.getDatagramSocket() instanceof MulticastSocket) {
            if (!(t instanceof Integer)) {
                throw new IllegalArgumentException("not an integer");
            }
            this.setTimeToLive((int)t);
        }
        else if (socketOption == StandardSocketOptions.IP_MULTICAST_LOOP && this.getDatagramSocket() instanceof MulticastSocket) {
            this.setOption(18, t);
        }
        else {
            throw new UnsupportedOperationException("unsupported option");
        }
    }
    
     <T> T getOption(SocketOption<T> socketOption) throws IOException {
        if (socketOption == StandardSocketOptions.SO_SNDBUF) {
            return this.getOption(4097);
        }
        if (socketOption == StandardSocketOptions.SO_RCVBUF) {
            return this.getOption(4098);
        }
        if (socketOption == StandardSocketOptions.SO_REUSEADDR) {
            return this.getOption(4);
        }
        if (socketOption == StandardSocketOptions.IP_TOS) {
            return this.getOption(3);
        }
        if (socketOption == StandardSocketOptions.IP_MULTICAST_IF && this.getDatagramSocket() instanceof MulticastSocket) {
            return this.getOption(31);
        }
        if (socketOption == StandardSocketOptions.IP_MULTICAST_TTL && this.getDatagramSocket() instanceof MulticastSocket) {
            return Integer.valueOf(this.getTimeToLive());
        }
        if (socketOption == StandardSocketOptions.IP_MULTICAST_LOOP && this.getDatagramSocket() instanceof MulticastSocket) {
            return this.getOption(18);
        }
        throw new UnsupportedOperationException("unsupported option");
    }
    
    protected FileDescriptor getFileDescriptor() {
        return this.fd;
    }
}
